@@ -26,10 +26,11 @@ import gzip
 
 from time import time
 
-i18n_catalog = i18nCatalog("cura")
-
+from time import gmtime
 from enum import IntEnum
 
+i18n_catalog = i18nCatalog("cura")
+
 class AuthState(IntEnum):
     NotAuthenticated = 1
     AuthenticationRequested = 2
@@ -101,6 +102,8 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
         self._target_bed_temperature = 0
         self._processing_preheat_requests = True
 
+        self._can_control_manually = False
+
         self.setPriority(3) # Make sure the output device gets selected above local file output
         self.setName(key)
         self.setShortDescription(i18n_catalog.i18nc("@action:button Preceded by 'Ready to'.", "Print over network"))
@@ -669,10 +672,11 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
             Logger.log("d", "Attempting to perform an action without authentication for printer %s. Auth state is %s", self._key, self._authentication_state)
             return
 
-        Application.getInstance().showPrintMonitor.emit(True)
+        Application.getInstance().getController().setActiveStage("MonitorStage")
         self._print_finished = True
         self.writeStarted.emit(self)
-        self._gcode = getattr(Application.getInstance().getController().getScene(), "gcode_list")
+        active_build_plate = Application.getInstance().getBuildPlateModel().activeBuildPlate
+        self._gcode = getattr(Application.getInstance().getController().getScene(), "gcode_dict")[active_build_plate]
 
         print_information = Application.getInstance().getPrintInformation()
         warnings = []  # There might be multiple things wrong. Keep a list of all the stuff we need to warn about.
@@ -723,10 +727,10 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
                                        remote_material_guid,
                                        material.getMetaDataEntry("GUID"))
 
-                            remote_materials = UM.Settings.ContainerRegistry.ContainerRegistry.getInstance().findInstanceContainers(type = "material", GUID = remote_material_guid, read_only = True)
+                            remote_materials = UM.Settings.ContainerRegistry.ContainerRegistry.getInstance().findInstanceContainersMetadata(type = "material", GUID = remote_material_guid, read_only = True)
                             remote_material_name = "Unknown"
                             if remote_materials:
-                                remote_material_name = remote_materials[0].getName()
+                                remote_material_name = remote_materials[0]["name"]
                             warnings.append(i18n_catalog.i18nc("@label", "Different material (Cura: {0}, Printer: {1}) selected for extruder {2}").format(material.getName(), remote_material_name, index + 1))
 
                     try:
@@ -764,7 +768,7 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
             if button == QMessageBox.Yes:
                 self.startPrint()
             else:
-                Application.getInstance().showPrintMonitor.emit(False)
+                Application.getInstance().getController().setActiveStage("PrepareStage")
         # For some unknown reason Cura on OSX will hang if we do the call back code
         # immediately without first returning and leaving QML's event system.
         QTimer.singleShot(100, delayedCallback)
@@ -774,6 +778,11 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
 
     ##  Start requesting data from printer
     def connect(self):
+        # Don't allow to connect to a printer with a faulty connection state.
+        # For instance when switching printers but the printer is disconnected from the network
+        if self._connection_state == ConnectionState.error:
+            return
+
         if self.isConnected():
             self.close()  # Close previous connection
 
@@ -842,7 +851,7 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
             self._write_finished = True  # post_reply does not always exist, so make sure we unblock writing
             if self._post_reply:
                 self._finalizePostReply()
-            Application.getInstance().showPrintMonitor.emit(False)
+        Application.getInstance().getController().setActiveStage("PrepareStage")
 
     ##  Attempt to start a new print.
     #   This function can fail to actually start a print due to not being authenticated or another print already
@@ -963,7 +972,8 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
 
     ##  Send all material profiles to the printer.
     def sendMaterialProfiles(self):
-        for container in UM.Settings.ContainerRegistry.ContainerRegistry.getInstance().findInstanceContainers(type = "material"):
+        registry = UM.Settings.ContainerRegistry.ContainerRegistry.getInstance()
+        for container in registry.findInstanceContainers(type = "material"):
             try:
                 xml_data = container.serialize()
                 if xml_data == "" or xml_data is None:
@@ -972,7 +982,7 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
                 names = ContainerManager.getInstance().getLinkedMaterials(container.getId())
                 if names:
                     # There are other materials that share this GUID.
-                    if not container.isReadOnly():
+                    if not registry.isReadOnly(container.getId()):
                         continue  # If it's not readonly, it's created by user, so skip it.
 
                 material_multi_part = QHttpMultiPart(QHttpMultiPart.FormDataType)
@@ -1137,6 +1147,11 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
                     else:
                         Logger.log("w", "Unable to save authentication for id %s and key %s", self._authentication_id, self._getSafeAuthKey())
 
+                    # Request 'system' printer data once, when we know we have authentication, so we know we can set the system time.
+                    url = QUrl("http://" + self._address + self._api_prefix + "system")
+                    system_data_request = QNetworkRequest(url)
+                    self._manager.get(system_data_request)
+
                 else:  # Got a response that we didn't expect, so something went wrong.
                     Logger.log("e", "While trying to authenticate, we got an unexpected response: %s", reply.attribute(QNetworkRequest.HttpStatusCodeAttribute))
                     self.setAuthenticationState(AuthState.NotAuthenticated)
@@ -1156,6 +1171,27 @@ class NetworkPrinterOutputDevice(PrinterOutputDevice):
                 else:
                     pass
 
+            elif self._api_prefix + "system" in reply_url:
+                # Check if the printer has time, and if this has a valid system time.
+                try:
+                    data = json.loads(bytes(reply.readAll()).decode("utf-8"))
+                except json.decoder.JSONDecodeError:
+                    Logger.log("w", "Received an invalid authentication request reply from printer: Not valid JSON.")
+                    return
+                if "time" in data and "utc" in data["time"]:
+                    try:
+                        printer_time = gmtime(float(data["time"]["utc"]))
+                        Logger.log("i", "Printer has system time of: %s", str(printer_time))
+                    except ValueError:
+                        printer_time = None
+                    if printer_time is not None and printer_time.tm_year < 1990:
+                        # The system time is not valid, sync our current system time to it, so we at least have some reasonable time in the printer.
+                        Logger.log("w", "Printer system time invalid, setting system time")
+                        url = QUrl("http://" + self._address + self._api_prefix + "system/time/utc")
+                        put_request = QNetworkRequest(url)
+                        put_request.setHeader(QNetworkRequest.ContentTypeHeader, "application/json")
+                        self._manager.put(put_request, str(time()).encode())
+
         elif reply.operation() == QNetworkAccessManager.PostOperation:
             if "/auth/request" in reply_url:
                 # We got a response to requesting authentication.
